import { AnyType, FunctionType } from './utils';
import { Output, Input } from './Type';
export declare type SyncFunctionValidator<T = AnyType> = FunctionType<T>;
export declare type AsyncFunctionValidator<T = AnyType> = FunctionType<PromiseLike<T>>;
export declare type FunctionValidator<T = AnyType> = FunctionType<PromiseLike<T> | T>;
export declare type Schema<T = AnyType> = T extends FunctionType ? FunctionValidator<T> : T extends object ? {
    [K in keyof T]: Schema<T[K]>;
} : FunctionValidator<T> | T;
export declare type SyncSchema<T> = T extends FunctionType ? SyncFunctionValidator<T> : T extends object ? {
    [K in keyof T]: SyncSchema<T[K]>;
} : SyncFunctionValidator<T> | T;
export declare type ValidatorOutput<T> = T extends SyncSchema<Output<T>> ? Output<T> : PromiseLike<Output<T>>;
export declare type SchemaValidator<T, I extends Array<AnyType> = [Input<T>]> = FunctionType<ValidatorOutput<T>, I>;
export default function Validate<T extends Schema>(schema: T, errorMsg?: string): SchemaValidator<T>;
